      SUBROUTINE QPYROBO(XI,YI,ZI,TI,THE,PHI,BEX,BEY,BEZ,XP,YP,ZP,TP)
C     N. Armesto, 16.04.2009
C     performs a boost and rotation of (t,x,y,z) to (tp,xp,yp,zp):
C     cut version of PYROBO, angles and boost parameters identical.
C
C     Note: to perform the transformation from the cms of the collision to
C     the cms of the hard scattering (as done at the beginning of the
C     shower in pyshow or in qpygin), use the following sequence:
C     - qpyrobo with arguments 0.d0,0.d0,bbx,bby,bbz.
C     - qpyrobo with arguments 0.d0,aa2,0.d0,0.d0,0.d0.
C     - qpyrobo with arguments aa1,0.d0,0.d0,0.d0,0.d0.
C     For the inverse transformation (as done at the end of pyshow or in
C     qpygeo), use a single call to qpyrobo with arguments
C     -aa1,-aa2,-bbx,-bby,-bbz.
C
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)
C...Local arrays.
      DIMENSION ROT(3,3),VR(3),DV(4)
C
      X=XI
      Y=YI
      Z=ZI
      T=TI
C...Rotate, typically from z axis to direction (theta,phi).
      IF(THE**2+PHI**2.GT.1D-20) THEN
        ROT(1,1)=COS(THE)*COS(PHI)
        ROT(1,2)=-SIN(PHI)
        ROT(1,3)=SIN(THE)*COS(PHI)
        ROT(2,1)=COS(THE)*SIN(PHI)
        ROT(2,2)=COS(PHI)
        ROT(2,3)=SIN(THE)*SIN(PHI)
        ROT(3,1)=-SIN(THE)
        ROT(3,2)=0D0
        ROT(3,3)=COS(THE)
C   Instead of loop 120 in PYROBO.
        VR(1)=X
        VR(2)=Y
        VR(3)=Z
C   Instead of loop 130 in PYROBO.
        J=1
        X=ROT(J,1)*VR(1)+ROT(J,2)*VR(2)+ROT(J,3)*VR(3)
        J=2
        Y=ROT(J,1)*VR(1)+ROT(J,2)*VR(2)+ROT(J,3)*VR(3)
        J=3
        Z=ROT(J,1)*VR(1)+ROT(J,2)*VR(2)+ROT(J,3)*VR(3)
      ENDIF
C   If nothing happens...
      XP=X
      YP=Y
      ZP=Z
      TP=T
C...Boost, typically from rest to momentum/energy=beta.
      IF(BEX**2+BEY**2+BEZ**2.GT.1D-20) THEN
        DBX=BEX
        DBY=BEY
        DBZ=BEZ
        DB=SQRT(DBX**2+DBY**2+DBZ**2)
        EPS1=1D0-1D-12
        IF(DB.GT.EPS1) THEN
C...Rescale boost vector if too close to unity.
          CALL PYERRM(3,'(PYROBO:) boost vector too large')
          DBX=DBX*(EPS1/DB)
          DBY=DBY*(EPS1/DB)
          DBZ=DBZ*(EPS1/DB)
          DB=EPS1
        ENDIF
        DGA=1D0/SQRT(1D0-DB**2)
C    Instead of loop 150 in PYROBO.
        DV(1)=X
        DV(2)=Y
        DV(3)=Z
        DV(4)=T
        DBV=DBX*DV(1)+DBY*DV(2)+DBZ*DV(3)
        DGABV=DGA*(DGA*DBV/(1D0+DGA)+DV(4))
        XP=DV(1)+DGABV*DBX
        YP=DV(2)+DGABV*DBY
        ZP=DV(3)+DGABV*DBZ
        TP=DGA*(DV(4)+DBV)
      ENDIF
C 
      RETURN
      END
